@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001-2003 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002, 2003 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -14,8 +14,6 @@
 """Python Object Publisher -- Publish Python objects on web servers
 
 Provide an apply-like facility that works with any mapping object
-
-$Id: publish.py 105429 2009-11-02 07:47:51Z ctheune $
 """
 import sys
 from zope import component
@@ -170,7 +168,12 @@ def publish(request, handle_errors=True):
                             retryException.getOriginalException(), False)
                         break
                     else:
-                        raise
+                        to_raise = retryException.getOriginalException()
+                        if to_raise is None:
+                            # There is no original exception inside
+                            # the Retry, so just reraise it.
+                            raise
+                        break
 
             except:
                 # Bad exception handler or retry method.
