@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001-2002 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,12 +12,10 @@
 #
 ##############################################################################
 """Default View Tests
-
-$Id$
 """
 import unittest
+from doctest import DocTestSuite
 from zope.testing.cleanup import cleanUp
-from zope.testing.doctestunit import DocTestSuite
 
 def cleanUpDoc(args):
     cleanUp()
