@@ -1,7 +1,7 @@
 # -*- coding: latin-1 -*-
 ##############################################################################
 #
-# Copyright (c) 2001-2002 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -13,21 +13,21 @@
 #
 ##############################################################################
 """HTTP Publisher Tests
-
-$Id: test_http.py 105429 2009-11-02 07:47:51Z ctheune $
 """
 import sys
 import tempfile
 import unittest
 from cStringIO import StringIO
 from Cookie import CookieError
+from doctest import DocFileSuite
 
 import zope.event
 import zope.testing.cleanup
 from zope.component import provideAdapter
-from zope.testing import doctest
 from zope.i18n.interfaces.locales import ILocale
 from zope.interface.verify import verifyObject
+from zope.security.checker import ProxyFactory
+from zope.security.proxy import removeSecurityProxy
 
 from zope.interface import implements
 from zope.publisher.interfaces.logginginfo import ILoggingInfo
@@ -286,7 +286,8 @@ class HTTPTests(unittest.TestCase):
         request = self._createRequest(env, '')
         request.response.redirect(request.URL)
         self.assertEquals(request.response.getStatus(), 303)
-        self.assertEquals(request.response.getHeader('location'), str(request.URL))
+        self.assertEquals(request.response.getHeader('location'),
+                          str(request.URL))
 
     def testUntrustedRedirect(self):
         # Redirects are by default only allowed to target the same host as the
@@ -596,6 +597,31 @@ class HTTPTests(unittest.TestCase):
         self.assertEquals(req._traversed_names, ['item'])
         self.assertEquals(req._vh_root, self.app.folder)
 
+    def test_traverseDuplicateHooks(self):
+        """
+        BaseRequest.traverse should not call traversal hooks on elements
+        previously traversed but wrapped in a security Proxy.
+        """
+        hooks = []
+        class HookPublication(DefaultPublication):
+
+            def callTraversalHooks(self, request, object):
+                hooks.append(object)
+
+            def traverseName(self, request, ob, name, check_auth=1):
+                # Fake the virtual host
+                if name == "vh":
+                    return ProxyFactory(ob)
+                return super(HookPublication, self).traverseName(
+                    request, removeSecurityProxy(ob), name, check_auth=1)
+
+        publication = HookPublication(self.app)
+        req = self._createRequest()
+        req.setPublication(publication)
+        req.setTraversalStack(req.getTraversalStack() + ["vh"])
+        req.traverse(self.app)
+        self.assertEquals(len(hooks), 3)
+
     def testInterface(self):
         from zope.publisher.interfaces.http import IHTTPCredentials
         from zope.publisher.interfaces.http import IHTTPApplicationRequest
@@ -724,7 +750,8 @@ class TestHTTPResponse(unittest.TestCase):
         headers, body = self._parseResult(response)
         # Check that the data have been written, and that the header
         # has been preserved
-        self.assertEqual(headers['Content-Type'], 'text/plain;charset=us-ascii')
+        self.assertEqual(headers['Content-Type'],
+                         'text/plain;charset=us-ascii')
         self.assertEqual(body, data)
 
         # Make sure that no Content-Length header was added
@@ -766,6 +793,39 @@ class TestHTTPResponse(unittest.TestCase):
         eq("text/plain;charset=cp1251", headers["Content-Type"])
         eq("test", body)
 
+        # see https://bugs.launchpad.net/zope.publisher/+bug/98395
+        # RFC 3023 types and */*+xml output as unicode
+
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
+            {"content-type": "text/xml"})
+        eq("text/xml;charset=utf-8", headers["Content-Type"])
+        eq("test", body)
+
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
+            {"content-type": "application/xml"})
+        eq("application/xml;charset=utf-8", headers["Content-Type"])
+        eq("test", body)
+
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
+            {"content-type": "text/xml-external-parsed-entity"})
+        eq("text/xml-external-parsed-entity;charset=utf-8",
+           headers["Content-Type"])
+        eq("test", body)
+
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
+            {"content-type": "application/xml-external-parsed-entity"})
+        eq("application/xml-external-parsed-entity;charset=utf-8",
+           headers["Content-Type"])
+        eq("test", body)
+
+        # Mozilla XUL
+        headers, body = self._getResultFromResponse(u"test", "utf-8",
+            {"content-type": "application/vnd+xml"})
+        eq("application/vnd+xml;charset=utf-8", headers["Content-Type"])
+        eq("test", body)
+
+        # end RFC 3023 / xml as unicode
+
         headers, body = self._getResultFromResponse("test", "utf-8",
             {"content-type": "image/gif"})
         eq("image/gif", headers["Content-Type"])
@@ -786,7 +846,8 @@ class TestHTTPResponse(unittest.TestCase):
         c = self._getCookieFromResponse([
                 ('foo', 'bar', {}),
                 ])
-        self.failUnless('foo=bar;' in c or 'foo=bar' in c, 'foo=bar; not in %r' % c)
+        self.failUnless('foo=bar;' in c or 'foo=bar' in c,
+                        'foo=bar; not in %r' % c)
 
         c = self._getCookieFromResponse([
                 ('foo', 'bar', {}),
@@ -798,7 +859,8 @@ class TestHTTPResponse(unittest.TestCase):
         c = self._getCookieFromResponse([
                 ('sign', u'\N{BIOHAZARD SIGN}', {}),
                 ])
-        self.failUnless((r'sign="\342\230\243";' in c) or (r'sign="\342\230\243"' in c))
+        self.failUnless((r'sign="\342\230\243";' in c) or
+                        (r'sign="\342\230\243"' in c))
 
         self.assertRaises(
                 CookieError,
@@ -840,7 +902,8 @@ class TestHTTPResponse(unittest.TestCase):
             "text/html;charset=utf-8")
         self.assertEquals(response.getStatus(), 500)
         self.assert_(response.consumeBody() in
-            ["<html><head><title>&lt;type 'exceptions.ValueError'&gt;</title></head>\n"
+            ["<html><head>"
+               "<title>&lt;type 'exceptions.ValueError'&gt;</title></head>\n"
             "<body><h2>&lt;type 'exceptions.ValueError'&gt;</h2>\n"
             "A server error occurred.\n"
             "</body></html>\n",
@@ -909,7 +972,7 @@ def test_suite():
     suite.addTest(unittest.makeSuite(ConcreteHTTPTests))
     suite.addTest(unittest.makeSuite(TestHTTPResponse))
     suite.addTest(unittest.makeSuite(HTTPInputStreamTests))
-    suite.addTest(doctest.DocFileSuite(
+    suite.addTest(DocFileSuite(
         '../httpresults.txt', setUp=cleanUp, tearDown=cleanUp))
     suite.addTest(unittest.makeSuite(APITests))
     return suite
