@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2008 Zope Foundation and Contributors.
+# Copyright (c) Zope Corporation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -12,7 +12,8 @@
 #
 ##############################################################################
 import re, unittest
-from zope.testing import doctest, renormalizing
+import doctest
+from zope.testing import renormalizing
 
 class SamplePublication:
 
